/*
    scconfig - c99 feature detection
    Copyright (C) 2009  Tibor Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

		Project page: http://repo.hu/projects/scconfig
		Contact via email: scconfig [at] igor2.repo.hu
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "libs.h"
#include "log.h"
#include "db.h"
#include "dep.h"
#include "find_mfunc_cc.h"

static int try(int logdepth, const char *prefix, const char *test_c, char *cflags, char *ldflags)
{
	char *out = NULL;
	char *tmp;

	logprintf(logdepth, "trying '%s' and '%s'\n", cflags, ldflags);
	if (compile_run(logdepth+1, test_c, NULL, cflags, ldflags, &out) == 0) {
		if (target_emu_fail(out) || (strncmp(out, "OK", 2) == 0)) {
			free(out);

			if (*cflags == '+')
				cflags++;
			if (*ldflags == '+')
				ldflags++;

			tmp = malloc(strlen(prefix) + 32);
			sprintf(tmp, "%s/cflags", prefix);
			put(tmp, cflags);
			sprintf(tmp, "%s/ldflags", prefix);
			put(tmp, ldflags);
			report("OK ('%s' and '%s')\n", cflags, ldflags);
			sprintf(tmp, "%s/present", prefix);
			put(tmp, strue);
			free(tmp);
			return 1;
		}
		free(out);
	}
	return 0;
}


int find_math_minpack(int logdepth, int fatal)
{
	char *test_c =
		NL "#include <minpack.h>"
		NL "int main() {"
		NL "	int one=1;"
		NL "	if (dpmpar_(&one) != 0.0)"
		NL "		puts(\"OK\");"
		NL "	return 0;"
		NL "}"
		NL;

	require("cc/cc", logdepth, fatal);

	report("Checking for minpack... ");
	logprintf(logdepth, "find_math_minpack: trying to find minpack...\n");
	logdepth++;

	/* Look at the standard places */
	if (try(logdepth, "libs/math/minpack", test_c, "", "+-lminpack")) return 0;

	report("Not found\n");
	put("libs/math/minpack/present", sfalse);
	return 1;
}


void deps_math_init()
{
	dep_add("libs/math/minpack/*",  find_math_minpack);
	dep_add("libs/math/cc/log/*",   find_math_cc_log);
}
