#include <libporty/config.h>
#include "os_includes.h"

/* UNIX */
#if defined(UNIX)

/* get netdb.h to declare addrinfo on glibc */
#ifndef __USE_POSIX
# define __USE_POSIX
#endif

#	define P_net_socket int
#	define P_NET_INVALID_SOCKET -1
#	define GET_LAST_ERROR() (errno)

	typedef struct ifreq IFREQ;
#	include <errno.h>
#	include <netdb.h>

#	if defined(__GLIBC__) && (__GLIBC__ <= 2) && (__GLIBC_MINOR__ <= 1)
		typedef uint32_t in_addr_t;
#	endif
#endif
/* UNIX */


/* WIN32 */
#if defined(WIN32)

#	define P_net_socket SOCKET
#	define P_NET_INVALID_SOCKET -1

# include <stdio.h>

#	if !(defined(__MINGW32__) || defined(__CYGWIN__))
		typedef SSIZE_T ssize_t;
		typedef int socklen_t;
#	endif

#	define GET_LAST_ERROR() WSAGetLastError()
#	define EWOULDBLOCK WSAEWOULDBLOCK

	typedef unsigned long in_addr_t;
	typedef INTERFACE_INFO IFREQ;
#endif
/* WIN32 */

#ifndef P_HAVE_IOCTLSOCKET
#	define P_ioctl_socket ioctl
#else
#	define P_ioctl_socket ioctlsocket
#endif

/* os_net_inprogress: an expression to check for "in progress" after a call (so it's not returned as an error) */
#if defined(UNIX)
#	define P_net_inprogress	((errno == EINPROGRESS) || (errno == EAGAIN))
#elif defined(WIN32)
#	define P_net_inprogress	((WSAGetLastError() == WSAEINPROGRESS) || (WSAGetLastError() == WSAEWOULDBLOCK))
#else
#	error einprogress not ported
#endif


/* === P_poll() start === */
#ifdef P_NET_POLL

/* use native poll */
#define P_POLLIN     POLLIN
#define P_POLLPRI    POLLPRI
#define P_POLLOUT    POLLOUT
#define P_POLLERR    POLLERR
#define P_POLLHUP    POLLHUP
#define P_POLLNVAL   POLLNVAL

#define P_poll poll
#define P_pollfd pollfd

#else
/* define our own struct and constants */
struct P_pollfd {
	int fd;
	short events;
	short revents;
};

#define P_POLLIN     0x001
#define P_POLLPRI    0x002
#define P_POLLOUT    0x004
#define P_POLLERR    0x008
#define P_POLLHUP    0x010
#define P_POLLNVAL   0x020

typedef unsigned long int nfds_t;

# ifdef P_NET_SELECT
#  define P_poll P_poll_select
int P_poll_select(struct P_pollfd *fds, nfds_t nfds, int timeout);
# else
#  define P_poll P_poll_busyloop
int P_poll_busyloop(struct P_pollfd *fds, nfds_t nfds, int timeout);
# endif
#endif
/* === P_poll() end === */

#ifdef P_HAVE_SHUT
#	define P_SHUT_RD SHUT_RD
#	define P_SHUT_WR SHUT_WR
#	define P_SHUT_RDWR SHUT_RDWR
#else
#	ifdef P_HAVE_SD
#		define P_SHUT_RD 0
#		define P_SHUT_WR 1
#		define P_SHUT_RDWR 2
#	else
#		define P_SHUT_RD SD_RECEIVE
#		define P_SHUT_WR SD_SEND
#		define P_SHUT_RDWR SD_BOTH
#	endif
#endif

#ifdef WIN32
P_net_socket P_socket(int af, int protocol, int type);
#else
#define P_socket socket
#endif
