#ifndef P_BUF_H
#define P_BUF_H

typedef struct P_buf_s P_buf_t;

struct P_buf_s {
	long int alloced;    /* allocated length of buf[] */
	long int used;       /* used length of buf[] */
	char *sep;           /* master if not NULL */
	P_buf_t *last;       /* master: last buff in the chain, for filling */
	P_buf_t *next, *prev;
	char buf[1];
};

/* Creates a new line buffer; input will be split at any of the sep chars */
P_buf_t *P_buf_init(const char *sep);

/* Append some more content to buff; content is \0 terminated string */
void P_buf_fill(P_buf_t *buff, const char *strin);

/* Append some more content to buff; content is a binary, len must be
   1 or larger. */
void P_buf_fillbin(P_buf_t *buff, const char *in, int len);

/* Return next "line" of buff; the caller may change the content of
   returned string and use the string even after subsequent P_buf_get
   calls. After use, the string must be free'd using P_buf_free. */
char *P_buf_get(P_buf_t *buff);

/* Same as P_buf_get but also returns length of the item (useful for
   binary buffers) */
char *P_buf_getbin(P_buf_t *buff, int *len);

/* free a single "line" returned by P_buf_get */
void P_buf_free(char *str);

/* return the number of bytes in the currently growing buffer - useful
   for limiting buffer size */
int P_buf_get_current_len(P_buf_t *buff);

/* discard and free all pending full and partial lines in buffer */
void P_buf_uninit(P_buf_t *buff);

#endif
